/**

	BrainFuck Compiler
	
	File			:	Compiler Code

	Created by		:	Reznichenko Anton aka AlterVision
						mailto:me@av13.ru
						http://www.av13.ru

	Project started :	29 november 2008 - 01:14
	Last edited		:	29 november 2008 - 02:55

	Describtion		:
		
		This is the first, and I hope, the only version of my brainfuck compiler.

		The brainfuck language is an esoteric programming language noted 
		for its extreme minimalism. It was designed to challenge and amuse 
		programmers, and is not suitable for practical use. 

		For more information, read: 
			All info	:	http://www.av13.ru/arts/brainfuck/
			Language	:	http://www.av13.ru/arts/brainfuck/#language
			Examples	:	http://www.av13.ru/arts/brainfuck/#examples

		Great thanks at all to Gensek (Ostrecov D.) for his advices, beer and 
		my girlfriend for the inspiration. Without you it would be impossible.

	License				:	GNU GLP
	License notice		:	Please, do not use this product! It can cause 
							possible injures and brain problems!
							I told you!!						

*/

#include <stdio.h>
#include <stdlib.h>
#include "compiler.h"
//#include "mc.h"

int brainfuck ( char * file ) {

	// Defining Main Variables
	FILE	*		flptr;			// File 
	brainfuck_t		bf;				// Context
	char			in;				// Input Char
	int				i, j;			// Temp Ints
	stack_p			stack_temp;		// Temp Stack Pointer

	// Token List 
	char			tokens[] = {
		TK_NEXT, TK_PREV, TK_IN, TK_OUT, TK_INC, TK_DEC, TK_STR, TK_END
	};

	// Initializing Context Vars
	bf.commands = 0;
	bf.pos		= 0;
	bf.ptr		= 0;
	bf.stack	= null;

	// Clearing Context Data
	for ( i = 0; i < BRAINFUCKS; i++ ) {
		bf.code[i]	= 0;
		bf.data[i]	= 0;
	}

	// Loading Code
	flptr = fopen( file, "r" );
	if ( flptr != null) {
		j = 0;
		while ( (in = fgetc(flptr))!= EOF ) {
			// Do not load more than (big number) commands
			if ( j >= BRAINFUCKS ) break;
			for ( i = 0; i < TK_COUNT; i++ ) {
				// Check if it is command not comment
				if ( in == tokens[i] ) {
					bf.code[bf.commands] = in;
					bf.commands ++;
					j ++;
					break;
				}
			}
		}
	} else {
		printf( "Fatal: file not found!\n" );
		return 1;
	}
	fclose( flptr );

	// Start Program Cycle
	while ( bf.pos < bf.commands ) {

		// Check Command Code
		switch ( bf.code[bf.pos] ) {

			// Go To Next Cell ( > )
			case TK_NEXT:
				bf.ptr = (bf.ptr + 1) % BRAINFUCKS;
				break;
			// Go To Prev Cell ( < )
			case TK_PREV:
				bf.ptr --;
				if (bf.ptr < 0) bf.ptr = 0;
				break;

			// Inc Cell Value ( + )
			case TK_INC:
				bf.data[bf.ptr] = (bf.data[bf.ptr] + 1) % 256;
				break;

			// Del Cell Value ( - )
			case TK_DEC:
				bf.data[bf.ptr] --;
				if ( bf.data[bf.ptr] < 0 ) bf.data[bf.ptr] = 0;
				break;
			
			// Input Char from StdIn ( , )
			case TK_IN:
				bf.data[bf.ptr] = getchar();
				break;

			// Print Char to StrOut ( . )
			case TK_OUT:
				putchar(bf.data[bf.ptr]);
				break;

			// Cycle Start ( [ )
			case TK_STR:
				if ( bf.data[bf.ptr] > 0) {
					// Start Cycle if *p != 0
					stack_temp = new stack_t;
//					stack_temp = (stack_p) mc_alloc (sizeof(stack_t));
					stack_temp->next = bf.stack;
					stack_temp->command = bf.pos;
					bf.stack = stack_temp;
				} else {
					// Go After The ] if *p == 0
					while ( bf.pos < bf.commands ) {
						if ( bf.code[bf.pos] == TK_END ) {
							break;
						} else {
							bf.pos ++;
						}
					}
				}
				break;

			// Cycle End ( ] )
			case TK_END:
				// Go Back Only If *p != 0							
				if ( bf.data[bf.ptr] > 0 ) {
					if (bf.stack != null) {
						stack_temp = bf.stack;
						bf.stack = bf.stack->next;
						bf.pos = stack_temp->command - 1;
						delete stack_temp;
//						mc_delete (stack_temp);
					} else {
						printf( "Fatal: stack is empty when ] is called on %d command\n", bf.pos);
						bf.pos = bf.commands;
					}
				}
				break;
		}

		// Next Command
		bf.pos ++;

	}

	// Clearing the Stack if Necessary
	while (bf.stack != null) {
		stack_temp = bf.stack;
		bf.stack = bf.stack->next;
		delete stack_temp;
//		mc_delete (stack_temp);
	}

	// That's All
	return 0;

}

// end. =)